from money import (
    Currency as MoneylibCurrency,
    Money as MoneylibMoney,
)

from .models import Rate


class Currency(MoneylibCurrency):
    def rate_to(self, related_currency):
        return Rate.objects.get(currency=self.code, related_currency=related_currency.code).value

    def back_rate_to(self, related_currency):
        return 1 / Rate.objects.get(currency=self.code, related_currency=related_currency.code).value

    def bid(self, related_currency):
        """It is an alias for the rate_to method"""
        return self.rate_to(related_currency)

    def ask(self, related_currency):
        """It is an alias for the back_rate_to method"""
        return self.back_rate_to(related_currency)


class Money(MoneylibMoney):
    def to_currency(self, related_currency):  # returns new Money object
        if related_currency == self.currency:
            return self
        rate = self.currency.rate_to(related_currency)
        return Money(self.amount * rate, related_currency)



class CurrenciesNotSpecifiedError(Exception):
    def __str__(self):
        return 'CURRENCIES must be specified'


class CurrencyDoesNotExist(Exception):
    def __init__(self, code):
        self.code = code

    def __str__(self):
        return 'Currency with code "%s" does not exist' % self.code


class MultipleCurrenciesFound(Exception):
    def __init__(self, code):
        self.code = code

    def __str__(self):
        return 'Multiple currencies found for code "%s"' % self.code



from django.db import models


class MoneyObjectsManager(models.Manager):
    def get_queryset(self):
        qs = super().get_queryset()
        return qs.select_related(*self.model.get_money_field_names())



from django.db import models
from django.conf import settings

from money import Money, Currency

from .mixins import MoneyObjectsManager


class Rate(models.Model):
    CURRENCY_CHOICES = ((c.code, c.name) for c in settings.CURRENCIES)

    currency = models.CharField(_('Currency'), max_length=8, choices=CURRENCY_CHOICES)
    related_currency = models.CharField(_('Related currency'), max_length=8, choices=CURRENCY_CHOICES)
    value = models.FloatField()

    def __str__(self):
        return f"{self.currency} to {self.related_currency}"


class MoneyModel(models.Model):
    amount = models.BigIntegerField()
    currency = models.CharField(max_length=8, choices=Rate.CURRENCY_CHOICES)

    @property
    def money(self):
        return Money.int_to_money(self.amount, Currency.get(self.currency))

    @money.setter
    def money(self, value):
        if isinstance(value, Money) and value.currency.code == self.currency:
            self.amount = int(value)
            self.save()
        else:
            raise TypeError("Must be Money instance with existed Currency")

    def __str__(self):
        return f'{self.money}'


class MoneyMixin(object):
    __money_fields = []

    objects = MoneyObjectsManager()

    def __init__(self, *args, **kwargs):
        self.__money_fields = [field for field in self._meta.fields if isinstance(field, MoneyField)]
        for field in self.__money_fields:
            money = kwargs.get(field.name)
            if money:
                money_instance = MoneyModel.objects.create(amount=int(money), currency=money.currency.code)
                kwargs.update({field.name: money_instance})
        super().__init__(*args, **kwargs)

    @classmethod
    def get_money_field_names(cls):
        return [field.name for field in cls._meta.fields if isinstance(field, MoneyField)]



from django.db import models

from money import Money, Currency

from .models import MoneyModel


class MoneyField(models.OneToOneField):
    def __init__(self, *args, **kwargs):
        kwargs['to'] = 'money.MoneyModel'
        super().__init__(*args, **kwargs)

    def get_internal_type(self):
        return "MoneyField"

    @staticmethod
    def parse_value(value):
        return Money.int_to_money(value.amount, Currency.get(value.currency))

    def from_db_value(self, value, expression, connection, *args, **kwargs):
        if value is None:
            return value
        if isinstance(value, int):
            return value
        raise TypeError

    def to_python(self, value):
        if isinstance(value, Money) or value is None:
            return value
        return self.parse_value(value)

    def get_db_prep_value(self, value, connection, prepared=False):
        if value is None:
            return value
        return MoneyModel.objects.create(amount=int(value), currency=value.currency.code)



# -*- coding: utf-8 -*-
import os
from setuptools import setup, find_packages

setup(
    name="django-moneylib",
    version=__import__('django_money').__version__,
    description=open(os.path.join(os.path.dirname(__file__), "DESCRIPTION")).read(),
    license="The MIT License (MIT)",
    keywords="django, money, django-money, moneylib, currency",

    author="Alexander Yudkin",
    author_email="san4ezy@gmail.com",

    maintainer="Alexander Yudkin",
    maintainer_email="san4ezy@gmail.com",

    url="https://github.com/san4ezy/django-moneylib",
    packages=find_packages(exclude=[]),
    install_requires=[],
    classifiers=[
        'Intended Audience :: Developers',
        'Environment :: Web Environment',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
    ],
)
